]> git.sesse.net Git - stockfish/blobdiff - src/tt.h
Rename TranspositionTable 'writes' in 'overwrites'
[stockfish] / src / tt.h
index 272e752b3157c6237fd62fb5fc3cc992cef86740..46f5b11dd3940864ace0da1c377f95aa8411735d 100644 (file)
--- a/src/tt.h
+++ b/src/tt.h
 class TTEntry {
 
 public:
-  TTEntry() {}
-  TTEntry(uint32_t k, Value v, ValueType t, Depth d, Move m, int generation)
-        : key_ (k), data((m & 0x1FFFF) | (t << 20) | (generation << 23)),
-          value_(int16_t(v)), depth_(int16_t(d)) {}
-
-  uint32_t key() const { return key_; }
-  Depth depth() const { return Depth(depth_); }
+  void save(uint32_t k, Value v, ValueType t, Depth d, Move m, int g, Value statV, Value kd) {
+
+      key32 = k;
+      data = (m & 0x1FFFF) | (t << 20) | (g << 23);
+      value16     = int16_t(v);
+      depth16     = int16_t(d);
+      staticValue = int16_t(statV);
+      kingDanger  = int16_t(kd);
+  }
+
+  uint32_t key() const { return key32; }
+  Depth depth() const { return Depth(depth16); }
   Move move() const { return Move(data & 0x1FFFF); }
-  Value value() const { return Value(value_); }
+  Value value() const { return Value(value16); }
   ValueType type() const { return ValueType((data >> 20) & 7); }
-  int generation() const { return (data >> 23); }
+  int generation() const { return data >> 23; }
+  Value static_value() const { return Value(staticValue); }
+  Value king_danger() const { return Value(kingDanger); }
 
 private:
-  uint32_t key_;
+  uint32_t key32;
   uint32_t data;
-  int16_t value_;
-  int16_t depth_;
+  int16_t value16;
+  int16_t depth16;
+  int16_t staticValue;
+  int16_t kingDanger;
 };
 
 
 /// This is the number of TTEntry slots for each position
-const int ClusterSize = 5;
+const int ClusterSize = 4;
 
 /// Each group of ClusterSize number of TTEntry form a TTCluster
-/// that is indexed by a single position key. Cluster is padded
-/// to a cache line size so to guarantee always aligned accesses.
+/// that is indexed by a single position key. TTCluster size must
+///  be not bigger then a cache line size, in case it is less then
+/// it should be padded to guarantee always aligned accesses.
 
 struct TTCluster {
   TTEntry data[ClusterSize];
-  char cache_line_padding[64 - sizeof(TTEntry[ClusterSize])];
 };
 
 
@@ -97,21 +106,19 @@ public:
   ~TranspositionTable();
   void set_size(size_t mbSize);
   void clear();
-  void store(const Key posKey, Value v, ValueType type, Depth d, Move m);
+  void store(const Key posKey, Value v, ValueType type, Depth d, Move m, Value statV, Value kingD);
   TTEntry* retrieve(const Key posKey) const;
-  void prefetch(const Key posKey) const;
   void new_search();
   void insert_pv(const Position& pos, Move pv[]);
-  void extract_pv(const Position& pos, Move pv[], const int PLY_MAX);
+  void extract_pv(const Position& pos, Move bestMove, Move pv[], const int PLY_MAX);
   int full() const;
+  TTEntry* first_entry(const Key posKey) const;
 
 private:
-  inline TTEntry* first_entry(const Key posKey) const;
-
-  // Be sure 'writes' is at least one cache line away
+  // Be sure 'overwrites' is at least one cache line away
   // from read only variables.
   unsigned char pad_before[64 - sizeof(unsigned)];
-  unsigned writes; // heavy SMP read/write access here
+  unsigned overwrites; // heavy SMP read/write access here
   unsigned char pad_after[64];
 
   size_t size;
@@ -121,4 +128,14 @@ private:
 
 extern TranspositionTable TT;
 
+
+/// TranspositionTable::first_entry returns a pointer to the first
+/// entry of a cluster given a position. The low 32 bits of the key
+/// are used to get the index in the table.
+
+inline TTEntry* TranspositionTable::first_entry(const Key posKey) const {
+
+  return entries[uint32_t(posKey) & (size - 1)].data;
+}
+
 #endif // !defined(TT_H_INCLUDED)